#include "StdAfx.h"
#include "mirror.h"
#include "polygon.h"
#include "universe.h"
#ifdef VC71
#else
#endif
#define _USE_MATH_DEFINES 
#include <math.h>
#ifdef VC71
#include <GL/glut.h>
#else
#include "glut.h"
#endif
#include <iostream>

extern bool RENDER_REFLECT;
extern bool RENDER_SHADOWS;

Mirror::Mirror(Vertex *point1, Vertex *point2, Universe *uni)
{
	//store the opposite corners of the mirror
	positionX = (*point1)(0);
	positionY = (*point1)(1);
	positionZ = (*point1)(2);

	x2 = (*point2)(0);
	y2 = (*point2)(1);
	z2 = (*point2)(2);

	//store a polygon for the mirror
	Polygon *polygon = new Polygon();
	polygon->addVertex(Vertex(positionX, positionY, positionZ), Vertex(0, 0, 0));
	polygon->addVertex(Vertex(x2, positionY, z2), Vertex(0, 1, 0));
	polygon->addVertex(Vertex(x2, y2, z2), Vertex(1, 1, 0));
	polygon->addVertex(Vertex(positionX, y2, positionZ), Vertex(1, 0, 0));

	rend = polygon;

	universe = uni;
}

Mirror::~Mirror(void)
{
}

void Mirror::render(void)
{
	//delegate if we should render reflectors
	if (RENDER_REFLECT)
		renderReflect();
	else
	{
		//render the mirror as a blank white polygon
		GLfloat colour[] = {1.0, 1.0, 1.0, 1.0};
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, colour);
		glDisable(GL_TEXTURE_2D);
		rend->render();
		glEnable(GL_TEXTURE_2D);

	}
}


void Mirror::renderReflect()
{
	//**render polygon to stencil buffer
	//disable writes to the colour buffer
	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	//enable the stencil buffer and draw the polygon of the mirror into the stencil buffer
	glEnable(GL_STENCIL_TEST);
	glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
	glStencilMask(0x80);
	glStencilFunc(GL_ALWAYS, 0x80, 0x80);
	rend->render();
	//draw 'holes' in the stencil buffer for any occluders
	glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
	glStencilFunc(GL_ALWAYS, 0x0, 0x80);
	universe->renderScene(false);
	//then clear the depth buffer (this is unfortunate)
	glClear(GL_DEPTH_BUFFER_BIT);

	//**flip world around the mirror
	glPushMatrix();
	float centreX = positionX + (x2-positionX)/2;
	float centreZ = positionZ + (z2-positionZ)/2;
	float dX = centreX - positionX;
	float dZ = centreZ - positionZ;
	float degRot = atan2(dX, -dZ) * 180/M_PI;
	//cout << "centreX: " << centreX <<' '<< "centreZ: " << centreZ <<'\n';
	//cout << "dX: " << dX <<' '<< "dZ: " << dZ <<'\n';
	//cout << "degRot: " << degRot <<'\n';

	//translate to centre
	glTranslatef(centreX, 0, centreZ);
	//rotate to align with z axis
	glRotatef(-degRot, 0, 1, 0);
	//flip around z-axis
	glScalef(-1, 1, 1);

	//clipping plane
	double planeEqn[] = {1, 0, 0, 0};
	glClipPlane(GL_CLIP_PLANE0, planeEqn);
	glEnable(GL_CLIP_PLANE0);

	//rotate back
	glRotatef(degRot, 0, 1, 0);
	//translate back
	glTranslatef(-centreX, 0, -centreZ);

	//**render world
	glCullFace(GL_FRONT);
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glStencilFunc(GL_EQUAL, 0x80, 0x80);
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
	glDepthMask(GL_TRUE);
	
	universe->renderNonReflect(true, true);

	glDisable(GL_CLIP_PLANE0);

	//**render polygon to depth buffer only
	glDisable(GL_STENCIL_TEST);
	glCullFace(GL_BACK);
	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
	rend->render();


	//**restore transform and depth test
	glPopMatrix();
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glDepthFunc(GL_LEQUAL);
}

void Mirror::init(void)
{
}

bool Mirror::reflects()
{
	//if reflection is turned on in the dimulation then this is a reflector
	return RENDER_REFLECT;
}
