#pragma once

#include "Vertex.h"
#include <vector>
#include "renderable.h"

using namespace std;

class ShadowVolume;


//a simple polygon
class Polygon : public Renderable
{
public:
	Polygon();
	virtual ~Polygon(void);

	//add a vertex to the polygon
	void addVertex(Vertex &vert, Vertex &tex)
	{
		vertexList.push_back(new Vertex(vert));
		texList.push_back(new Vertex(tex));
	}

	//sends this polygon to OpenGL
	void render();

	//renders this polygon's shadow volume
	void renderShadowVolumes(Vertex *lightPosition);

	//returns a normal vector to this polygon by calculating a cross product
	Vertex *getNormal(void);

	//returns the number of edges/points of the polygon
	unsigned int order(void)
	{
		return vertexList.size();
	}

	//calculate a normal vector to a polygon with the given corners
	static Vertex calcNormal(Vertex &v0, Vertex &v1, Vertex &v2, Vertex &v3);
	//calculate the plane equation for a polygon with the given vertices
	static void calculatePlaneEqn(Vertex &v0, Vertex &v1, Vertex &v2, double *result);

	//rotate the polygon about the y axis
	virtual void rotateY(float rads);

	//find the point at the center of the polygon
	Vertex getCenter();

private:
	//returns a pointer to the ith vertex in the polygon
	Vertex *getVertex(int i)
	{
		return vertexList[i];
	}

	//this polygon's normal vector
	Vertex *normal;

	//the edges that make up the polygon
	vector<Vertex *> vertexList;
	//tex co-ords for the polygon
	vector<Vertex *> texList;

	friend class ShadowVolume;
};
