#include "StdAfx.h"
#include "simplemap.h"
#include "Texture.h"
#include "TextureManager.h"
#ifdef VC71
#include <GL/glut.h>
#else
#include "glut.h"
#endif

SimpleMap::SimpleMap(float x1d, float x2d, float z1d, float z2d)
{
	//store size and textures
	x1 = x1d;
	x2 = x2d;
	z1 = z1d;
	z2 = z2d;

	walls = TextureManager::getInstance()->getTexture("../textures/bricksWall.tex");
	floor = TextureManager::getInstance()->getTexture("../textures/bricksFloor.tex");
}

SimpleMap::~SimpleMap(void)
{
	TextureManager::getInstance()->releaseTexture(floor);
	TextureManager::getInstance()->releaseTexture(walls);
}

bool SimpleMap::collide(Thing *thing)
{
	//very simple collision detection, just compare the user's position to that of
	//the walls

	bool collision = false;
	if (thing->getX() >= x2 - 1)
	{
		thing->setX(x2 - 1.1);
		collision = true;
	}
	if (thing->getX() <= x1 + 1)
	{
		thing->setX(x1 + 1.1);
		collision = true;
	}
	if (thing->getZ() >= z2 - 1)
	{
		thing->setZ(z2 - 1.1);
		collision = true;
	}
	if (thing->getZ() <= z1 + 1)
	{
		thing->setZ(z1 + 1.1);
		collision = true;
	}

	return collision;
}

void SimpleMap::render()
{
	floor->render();
	glBegin(GL_QUADS);
		//the floor
		glNormal3f(0, 1, 0);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(x1, 0, z1);
		glTexCoord2f(10.0f, 0.0f);
		glVertex3f(x1, 0, z2);
		glTexCoord2f(10.0f, 10.0f);
		glVertex3f(x2, 0, z2);
		glTexCoord2f(0.0f, 10.0f);
		glVertex3f(x2, 0, z1);
	glEnd();
	walls->render();
	glBegin(GL_QUADS);
		//wall 1
		glNormal3f(0, 0, 1);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(x1, 0, z1);
		glTexCoord2f(10.0f, 0.0f);
		glVertex3f(x2, 0, z1);
		glTexCoord2f(10.0f, 1.0f);
		glVertex3f(x2, 2, z1);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(x1, 2, z1);
		//wall 2
		glNormal3f(1, 0, 0);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(x2, 0, z1);
		glTexCoord2f(10.0f, 0.0f);
		glVertex3f(x2, 0, z2);
		glTexCoord2f(10.0f, 1.0f);
		glVertex3f(x2, 2, z2);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(x2, 2, z1);
		//wall 3
		glNormal3f(0, 0, -1);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(x2, 0, z2);
		glTexCoord2f(10.0f, 0.0f);
		glVertex3f(x1, 0, z2);
		glTexCoord2f(10.0f, 1.0f);
		glVertex3f(x1, 2, z2);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(x2, 2, z2);
		//wall 4
		glNormal3f(-1, 0, 0);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(x1, 0, z2);
		glTexCoord2f(10.0f, 0.0f);
		glVertex3f(x1, 0, z1);
		glTexCoord2f(10.0f, 1.0f);
		glVertex3f(x1, 2, z1);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(x1, 2, z2);
	glEnd();
}
