#pragma once

#include <vrj/Draw/OGL/GlContextData.h>

using namespace vrj;

//a texture to be texture mapped
//Texture objects should not be used directly, but should be used via
//the texture manager class

class Texture
{
public:
	//creates a new Texture from the given .tex file
	Texture(char *fileName);
	virtual ~Texture(void);

	//sets up opengl to draw a polygon with this texture
	void render(void);

	//count references to this object (used by TextureManager)
	//note that we are not counting the strict number of refs to this object
	//but the number of objects in the scene using this texture
	int getRefCount(void) { return refs; }
	void incRefCount(void) { ++refs; }
	void decRefCount(void) { --refs; }

	//the filename from which the texture was loaded
	char *getFileName(void) { return fName; }

	void loadToGL(void);

private:
	unsigned int width, height;
	unsigned char *image;

	GlContextData<unsigned int> textureId;

	//filename
	char *fName;

	//loads a texture from disk
	void loadImage(char *fileName);
	//loads this texture into texture memory
	void loadTexture(void);

	//reference count
	int refs;

};
