#pragma once
#include <map>
using namespace std;

class Texture;

//manages the textures, so if you have several objects with the same texture
//you only have one shared Texture object

//implements the singleton pattern

class TextureManager
{
public:
	~TextureManager(void);

	//get a texture of the given image file, if a Texture object exists already then it
	//is returned, otherwise a new Texture object is created and returned.
	Texture *getTexture(char *fileName);
	//call when we are done using a texture object
	void releaseTexture(Texture *tex);

	static TextureManager *getInstance(void)
	{
		static TextureManager instance;
		return &instance;
	}

	void loadTexturesToGL(void);

private:
	TextureManager(void);
	TextureManager(const TextureManager&);
    TextureManager& operator= (const TextureManager&);

	//store a map of filenames to texture objects
	map<char *, Texture *> textures;
};
