#pragma once
#include "Map.h"
#include "user.h"
#include <vector>

using namespace std;
class Camera;
class Light;
class Mirror;

//contains everything in the scene and handles the top level rendering stuff (passes etc)

class Universe
{
public:
	Universe(void);
	virtual ~Universe(void);

	//render the universe to screen
	virtual void render();
	//update all objects in the universe
	virtual void update(float elapsedSeconds);
	//render all non reflecting objects in the universe
	//showAvatar=true --> render the user's avatar
	//reflected=true --> the universe should be rendered as if it is a reflection
	void renderNonReflect(bool showAvatar, bool reflected);
	//initialise the universe and all objects in it
	virtual void init();

	//add an object to the universe
	void addThing(Thing *thing) { things.push_back(thing);}

	//collision detection
	//currently only handles collisions between the user and the map
	bool collide(Thing *thing) { return map->collide(thing); }

	//return the controller object for the universe
	Controller *getController() { return user; }

	//render everything physical (ie not camera/lights) in the universe to opengl
	void renderScene(bool showAvatar);

private:
	vector<Thing *> things;
	Map *map;
	::User *user;
	Light *light;

	//add n small boxes to the universe
	void addRandomBoxes(int n);
	//render the shadow volumes of all objects in the universe
	void renderShadowVolumes(void);
	//render fake shadows for all the objects
	void renderFakeShadows();
};
