#include "StdAfx.h"
#include "vertex.h"
#include <math.h>
#ifdef VC71
#include <iostream>
#else
#include <iostream.h>
#endif

//constructor
Vertex::Vertex(double x, double y, double z)
{
	storage[0] = x;
	storage[1] = y;
	storage[2] = z;
	storage[3] = 1;
}

//default constructor
Vertex::Vertex()
{
	storage[0] = 0;
	storage[1] = 0;
	storage[2] = 0;
	storage[3] = 1;
}

//translate by (x, y, z)
void Vertex::translate(double x, double y, double z)
{
	storage[0] += x;
	storage[1] += y;
	storage[2] += z;
}

//scale by (x, y, z)
void Vertex::scale(double x, double y, double z)
{
	storage[0] *= x;
	storage[1] *= y;
	storage[2] *= z;
}

//|a| = sqrt(ax2 + ay2 + az2 + aw2), aw = 0
double Vertex::size(void)
{
	return sqrt(storage[0]*storage[0] + storage[1]*storage[1] + storage[2]*storage[2]);
}

//produces a unit vector of the same direction as the original vector
void Vertex::normalise(void)
{
	storage[0] /= size();
	storage[1] /= size();
	storage[2] /= size();
	storage[3] = 0;
}

//calculates the cross product of two vectors
Vertex crossProduct(Vertex &v1, Vertex &v2)
{
	Vertex result(v1(1)*v2(2) - v2(1)*v1(2), v1(2)*v2(0)-v2(2)*v1(0), v1(0)*v2(1) - v2(0)*v1(1));

	return result;
}

double dotProduct(Vertex &v1, Vertex &v2)
{
	double result = v1(0)*v2(0) + v1(1)*v2(1) + v1(2)*v2(2);

	return result;
}

void Vertex::rotateY(float rads)
{
	rads *= -1;
	double x = storage[0];
	double z = storage[2];
	storage[0] = x*cos(rads) - z*sin(rads);
	storage[2] = x*sin(rads) + z*cos(rads);
}
